////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 3D bezier spline drawing example program.
// This program generates a closed spline with 12 control points based on spline data provided in the data text file.
// The program then moves a 3D object along the curve using roller coaster physics.
// The user can switch from the free-view camera to the object camera. Finally, a background skybox is rendered
// in order to give the user a sense of orientation.
//
// Note: The applied roller coaster physics only work with the default curve set. If you want to make them work 
// for other curve sets, then you have to adjust & calibrate the PE and KE levels at different slope gradients.
//
// Written by Vladeta Stojanovic (0602920@live.abertay.ac.uk).
//
// For a good reference on roller coaster physics, see: http://myphysicslab.com/RollerSimple.html
// For a good reference on Bezier spline generation and rendering (using DeCasteljau's Algorithm), see: http://www.cubic.org/docs/bezier.htm
// Skybox textures downloaded from: http://www.humus.name/
//
// Version: 281211_01
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <GL/glew.h>
#include <GL/freeglut.h>
#include <stdio.h>
#include <windows.h>
#include <mmsystem.h>
#include <math.h>
#include <iostream>
#include <fstream>
#include "glext.h"
#include "camera.h"
#include "timer.h"
#include "BCurve3D.h"
#include "teapot.h"
#include "tgaTex.h"
#include "BCurveMaths.h"

using namespace std;


///////////////////////////////////////////////////
//Global application object instance variables
///////////////////////////////////////////////////
Camera *gCamera; //camera

//Present curves
BCurve *gBCurve0;
BCurve *gBCurve1;
BCurve *gBCurve2;

//Future curves the present curves get lerped to
GLuint	Textures[6]; //skybox textures
Teapot *gTeapot; //teapot model
BCurve *gBCurve0t; //future curves the present curves get lerped to
BCurve *gBCurve1t;
BCurve *gBCurve2t;
float inputPoints0[12]; //read input points for curve0
float inputPoints1[12]; //read input points for curve0
float inputPoints2[12]; //read input points for curve0
GLfloatPoint mtPointSet0; //movement coords interpolated along the 1st curve
GLfloatPoint mtPointSet1; //movement coords interpolated along the 2nd curve
GLfloatPoint mtPointSet2; //movement coords interpolated along the 3rd curve

//////////////////////////////////////////////
//Global OpenGL rendering context variables
//////////////////////////////////////////////
GLenum    ePolygonMode = GL_FILL;
GLboolean bLight = true;

////////////////////////////////////////////////////////////////////////////
//Curve generation/interpolation/control global variable declerations
/////////////////////////////////////////////////////////////////////////////
int _time;
int lastTime;
int delta;
float stepTime0 = 0.0f; //step time for curve 0
float stepTime1 = 0.0f; //step time for curve 1
float stepTime2 = 0.0f; //step time for curve 2
float step = 0.0004f;
float t0 = 0.0f;
float t1 = 0.0f;
float t2 = 0.0f;
bool generate = false;
bool useObjectCam = false;
bool useFrontCam = false;
bool useGravity = true;
float k; //gradient interpolation constant
float theta;
float rot = 0; //first person veiw camera rotation angle

//defualt curve values read in from a text file
ifstream inFile0;
ifstream inFile1;
ifstream inFile2;

////////////////////////////////////
//Main scene function prototypes
////////////////////////////////////
void SetupCurves();
void SetupCurvesT();
void RenderStaticLight();
void Draw3DGrid();
void DrawSkyBox();
void DrawScene();
void IdleScene();
void ResizeWindow(GLsizei iWidth, GLsizei iHeight);
void KeyboardInput(unsigned char cKey, int iX, int iY);
void DebugInfo();

void DebugInfo()
{
	//stream any debug info via cout here
	system("CLS");

	cout << "spline movement x-coord: " << mtPointSet0.x << endl;
	cout << "spline movement y-coord: " << mtPointSet0.y << endl;
	cout << "spline movement z-coord: " << mtPointSet0.z << endl;
	cout << "stepTime0: " << stepTime0 << endl;
	cout << "stepTime1: " << stepTime1 << endl;
	cout << "stepTime2: " << stepTime2 << endl;
	cout << "dt: " << delta * step << endl;
	cout << "teapot.y: " << gTeapot->mY << endl;
	cout << "pointSet0.y: " << mtPointSet0.y << endl;
	cout << "mtPointSet0.y + dt: " << mtPointSet0.y + (delta * step) << endl;
	cout << "step: " << step << endl;
	cout << "k: " << k << endl;
	cout << "theta: " << Rad2Deg(theta) << endl;
	cout << "rot: " << rot << endl;

}

//////////////////////////////////////////////////
// Curve generation/interpolation functions
//////////////////////////////////////////////////
void ReadCurveData()
{
	inFile0.open ("data/curve0.txt");
	inFile1.open ("data/curve1.txt");
	inFile2.open ("data/curve2.txt");

	if (!inFile0 || !inFile1 || !inFile2)
	{
		cout << "could not open curve data file" << endl;
	}

	//read in default coords for curve0
	for(int i = 0; i < 12; ++i)
	{
		inFile0 >> inputPoints0[i];
	}
	inFile0.close();

	//read in default coords for curve1
	for(int j = 0; j < 12; ++j)
	{
		inFile1 >> inputPoints1[j];
	}
	inFile1.close();

	//read in default coords for curve2
	for(int k = 0; k < 12; ++k)
	{
		inFile2 >> inputPoints2[k];
	}
	inFile2.close();

}

void SetupCurves()
{
	gBCurve0->points[4] = gBCurve1->points[1];
	gBCurve1->points[4] = gBCurve2->points[1];
	gBCurve2->points[4] = gBCurve0->points[1];
}

void SetupCurvesT()
{
	gBCurve0t->points[4] = gBCurve1t->points[1];
	gBCurve1t->points[4] = gBCurve2t->points[1];
	gBCurve2t->points[4] = gBCurve0t->points[1];
}

void SetupDefaultCurve()
{
	//set curve0 data from read in text file data
	gBCurve0->points[1].x = inputPoints0[0];
	gBCurve0->points[1].y = inputPoints0[1];
	gBCurve0->points[1].z = inputPoints0[2];

	gBCurve0->points[2].x = inputPoints0[3];
	gBCurve0->points[2].y = inputPoints0[4];
	gBCurve0->points[2].z = inputPoints0[5];

	gBCurve0->points[3].x = inputPoints0[6];
	gBCurve0->points[3].y = inputPoints0[7];
	gBCurve0->points[3].z = inputPoints0[8];

	gBCurve0->points[4].x = inputPoints0[9];
	gBCurve0->points[4].y = inputPoints0[10];
	gBCurve0->points[4].z = inputPoints0[11];

	//set curve1 data from read in text file data
	gBCurve1->points[1].x = inputPoints1[0];
	gBCurve1->points[1].y = inputPoints1[1];
	gBCurve1->points[1].z = inputPoints1[2];

	gBCurve1->points[2].x = inputPoints1[3];
	gBCurve1->points[2].y = inputPoints1[4];
	gBCurve1->points[2].z = inputPoints1[5];

	gBCurve1->points[3].x = inputPoints1[6];
	gBCurve1->points[3].y = inputPoints1[7];
	gBCurve1->points[3].z = inputPoints1[8];

	gBCurve1->points[4].x = inputPoints1[9];
	gBCurve1->points[4].y = inputPoints1[10];
	gBCurve1->points[4].z = inputPoints1[11];

	
	//set curve2 data from read in text file data
	gBCurve2->points[1].x = inputPoints2[0];
	gBCurve2->points[1].y = inputPoints2[1];
	gBCurve2->points[1].z = inputPoints2[2];

	gBCurve2->points[2].x = inputPoints2[3];
	gBCurve2->points[2].y = inputPoints2[4];
	gBCurve2->points[2].z = inputPoints2[5];

	gBCurve2->points[3].x = inputPoints2[6];
	gBCurve2->points[3].y = inputPoints2[7];
	gBCurve2->points[3].z = inputPoints2[8];

	gBCurve2->points[4].x = inputPoints2[9];
	gBCurve2->points[4].y = inputPoints2[10];
	gBCurve2->points[4].z = inputPoints2[11];

	SetupCurves();
}

void SetupDefaultCurveT()
{
	//set curve0t data from read in text file data
	gBCurve0t->points[1].x = inputPoints0[0];
	gBCurve0t->points[1].y = inputPoints0[1];
	gBCurve0t->points[1].z = inputPoints0[2];

	gBCurve0t->points[2].x = inputPoints0[3];
	gBCurve0t->points[2].y = inputPoints0[4];
	gBCurve0t->points[2].z = inputPoints0[5];

	gBCurve0t->points[3].x = inputPoints0[6];
	gBCurve0t->points[3].y = inputPoints0[7];
	gBCurve0t->points[3].z = inputPoints0[8];

	gBCurve0t->points[4].x = inputPoints0[9];
	gBCurve0t->points[4].y = inputPoints0[10];
	gBCurve0t->points[4].z = inputPoints0[11];

	//set curve1t data from read in text file data
	gBCurve1t->points[1].x = inputPoints1[0];
	gBCurve1t->points[1].y = inputPoints1[1];
	gBCurve1t->points[1].z = inputPoints1[2];

	gBCurve1t->points[2].x = inputPoints1[3];
	gBCurve1t->points[2].y = inputPoints1[4];
	gBCurve1t->points[2].z = inputPoints1[5];

	gBCurve1t->points[3].x = inputPoints1[6];
	gBCurve1t->points[3].y = inputPoints1[7];
	gBCurve1t->points[3].z = inputPoints1[8];

	gBCurve1t->points[4].x = inputPoints1[9];
	gBCurve1t->points[4].y = inputPoints1[10];
	gBCurve1t->points[4].z = inputPoints1[11];

	
	//set curve2t data from read in text file data
	gBCurve2t->points[1].x = inputPoints2[0];
	gBCurve2t->points[1].y = inputPoints2[1];
	gBCurve2t->points[1].z = inputPoints2[2];

	gBCurve2t->points[2].x = inputPoints2[3];
	gBCurve2t->points[2].y = inputPoints2[4];
	gBCurve2t->points[2].z = inputPoints2[5];

	gBCurve2t->points[3].x = inputPoints2[6];
	gBCurve2t->points[3].y = inputPoints2[7];
	gBCurve2t->points[3].z = inputPoints2[8];

	gBCurve2t->points[4].x = inputPoints2[9];
	gBCurve2t->points[4].y = inputPoints2[10];
	gBCurve2t->points[4].z = inputPoints2[11];

	SetupCurvesT();
}

//Spline point cubic interpolation
GLfloatPoint PointOnSpline(BCurve *bSpline,  float t)
{
	float t0, t1, t2;
	GLfloatPoint tempPoint;

	t0 = 1 - t;
	t1 = powf(t0, 3);
	t2 = powf(t, 3);

	tempPoint.x = t1*bSpline->points[1].x + 3*t*t0*t0*bSpline->points[2].x + 3*t*t*t0*bSpline->points[3].x + t2*bSpline->points[4].x;
	tempPoint.y = t1*bSpline->points[1].y + 3*t*t0*t0*bSpline->points[2].y + 3*t*t*t0*bSpline->points[3].y + t2*bSpline->points[4].y;
	tempPoint.z = t1*bSpline->points[1].z + 3*t*t0*t0*bSpline->points[2].z + 3*t*t*t0*bSpline->points[3].z + t2*bSpline->points[4].z;

	return tempPoint;
}

//////////////////////////////////
//Physics functions
//////////////////////////////////

//Note: The trick with this is to ONLY mess about with the stepping value, and nothing else, based on the interpolated gradient value of K

void AddGravity(float dt)
{
	//add gravitational force
	step = Lerp((dt * (gTeapot->mMass * 9.8f)) * 0.01f, step, 0.001f);
}

void AddKineticEnergy(float dt)
{
	step = Lerp((dt * (0.5 * gTeapot->mMass * 9.8f)) * 0.01f, step, 0.00009f);
}

//Note: The phyics are adjusted to work with the default curve set only, as the gradient values of K
//never go over 2. For other randomly generated curves, this may not work too well. 
void CheckSlope(float dt)
{
	k = ((mtPointSet0.y + dt)  -  (step)) / ((mtPointSet0.x + dt)  -  (step));


	if(k > 2.5f)
	{
		step = Lerp((dt * (0.5 * gTeapot->mMass * 9.8f)) * 0.01f, step, 0.000001f); //PE
	}
	else if(k > 2.3f && k < 2.5f)
	{
		step = Lerp((dt * (0.5 * gTeapot->mMass * 9.8f)) * 0.01f, step, 0.00000005f); //PE
	}
	else if(k > 2.0f && k < 2.3f)
	{
		step = Lerp((dt * (0.5 * gTeapot->mMass * 9.8f)) * 0.01f, step, 0.00000003f); //PE
	}
	else if(k > 1.8f && k < 2.0f)
	{
		step = Lerp((dt * (0.5 * gTeapot->mMass * 9.8f)) * 0.01f, step, 0.00000002f); //PE
	}
	else if(k > 1.7f && k < 1.8f)
	{
		step = Lerp((dt * (0.5 * gTeapot->mMass * 9.8f)) * 0.01f, step, 0.000000015f); //PE
		rot = Lerp(0.5, -30, 30.0f);
	}
	else if(k > 1.6f && k < 1.7f)
	{
		step = Lerp((dt * (0.5 * gTeapot->mMass * 9.8f)) * 0.01f, step, 0.00000001f); //PE
	}
	else if(k > 1.4f && k < 1.6f)
	{
		step = Lerp((dt * (0.5 * gTeapot->mMass * 9.8f)) * 0.01f, step, 0.000000009f); //PE
	}
	else if(k > 1.2f && k < 1.4f)
	{
		step = Lerp((dt * (0.5 * gTeapot->mMass * 9.8f)) * 0.01f, step, 0.000000001f); //PE
	}
	else if(k > 1.0f && k < 1.1f)
	{
		step = Lerp((dt * (gTeapot->mMass * 9.8f)) * 0.01f, step, 0.0005f); //KE
		rot = Lerp(0.5, rot, -30.0f);
	}
	else if(k > 0.8f && k < 1.0f)
	{
		step = Lerp((dt * (gTeapot->mMass * 9.8f)) * 0.01f, step, 0.0006f); //KE
	}
	else if(k > 0.6f && k < 0.8f)
	{
		step = Lerp((dt * (gTeapot->mMass * 9.8f)) * 0.01f, step, 0.0009f); //KE
	}
	else if(k > 0.4f && k < 0.6f)
	{
		step = Lerp((dt * (gTeapot->mMass * 9.8f)) * 0.01f, step, 0.0007f); //KE
	}
	else if(k > 0.2f && k < 0.4f)
	{
		step = Lerp((dt * (gTeapot->mMass * 9.8f)) * 0.01f, step, 0.002f); //KE
	}
	else if(k < 0.2f)
	{
		step = Lerp((dt * (gTeapot->mMass * 9.8f)) * 0.01f, step, 0.0007f); //KE
	}
}

void CalculateSlopeRotation()
{
	float A;
	float B;

	A = -k;
	B = sqrtf(1 + powf(k, 2));

	//also, this is the correct rotation:
	//theta = powf(cosf(A / B), -1);
	//however, the "incorrect" rotation looks better

	theta = cosf(A / B);
}

////////////////////////////////////////////////
// Addtional scene rendering functionality.
////////////////////////////////////////////////
void RenderStaticLight()
{
	GLfloat Light_Position[] = {0.0f, -1000.0f, -200.0f, 1.0f};

	glPushMatrix();
	{
		glRotatef(Rad2Deg(theta), -1, 0, 1);
		glLightfv(GL_LIGHT0, GL_POSITION, Light_Position);
	}
	glPopMatrix();
}

void InitSkybox()
{
	//Load the textures
	CreateGLTexture("skybox/up.tga", Textures[0], true);
	CreateGLTexture("skybox/down.tga", Textures[1], true);
	CreateGLTexture("skybox/left.tga", Textures[2], true);
	CreateGLTexture("skybox/right.tga", Textures[3], true);
	CreateGLTexture("skybox/front.tga", Textures[4], true);
	CreateGLTexture("skybox/back.tga", Textures[5], true);

	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);// Linear Filtering
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);// Linear Filtering
}

void RenderSkybox()
{
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_CULL_FACE);	

	glPushMatrix();

	glLoadIdentity();

	gCamera->Look();

	glTranslatef(gCamera->GetPosX(), gCamera->GetPosY(),gCamera->GetPosZ());

	//Set the skybox color to white
	glColor3f(255.0f, 255.0f, 255.0f);

	//Skybox front
	glBindTexture(GL_TEXTURE_2D, Textures[5]);
	glBegin (GL_QUADS);
	{
		//Front face			
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(-10.0f, -10.0f,  10.0f);
		glTexCoord2f(1.0f, 0.0f); 
		glVertex3f( 10.0f, -10.0f,  10.0f);
		glTexCoord2f(1.0f, 1.0f); 
		glVertex3f( 10.0f,  10.0f,  10.0f);
		glTexCoord2f(0.0f, 1.0f); 
		glVertex3f(-10.0f,  10.0f,  10.0f);
	}
	glEnd();

	//Skybox back
	glBindTexture(GL_TEXTURE_2D, Textures[4]);
	glBegin (GL_QUADS);
	{			
		glTexCoord2f(1.0f, 0.0f); 
		glVertex3f(-10.0f, -10.0f, -10.0f);
		glTexCoord2f(1.0f, 1.0f); 
		glVertex3f(-10.0f,  10.0f, -10.0f);
		glTexCoord2f(0.0f, 1.0f); 
		glVertex3f( 10.0f,  10.0f, -10.0f);
		glTexCoord2f(0.0f, 0.0f); 
		glVertex3f( 10.0f, -10.0f, -10.0f);
	}
	glEnd();
	
	//Skybox top
	glBindTexture(GL_TEXTURE_2D, Textures[0]);
	glBegin (GL_QUADS);
	{					
		glTexCoord2f(0.0f, 1.0f); 
		glVertex3f(-10.0f,  10.0f, -10.0f);
		glTexCoord2f(0.0f, 0.0f); 
		glVertex3f(-10.0f,  10.0f,  10.0f);
		glTexCoord2f(1.0f, 0.0f); 
		glVertex3f( 10.0f,  10.0f,  10.0f);
		glTexCoord2f(1.0f, 1.0f); 
		glVertex3f( 10.0f,  10.0f, -10.0f);
	}
	glEnd();

	//Skybox bottom
	glBindTexture(GL_TEXTURE_2D, Textures[1]);

	glPushMatrix();
	glRotatef(180.0f, 0, 1, 0); // a little hack to get the texture to line up properly...
	glBegin (GL_QUADS);
	{				
		glTexCoord2f(1.0f, 1.0f); 
		glVertex3f(-10.0f, -10.0f, -10.0f);
		glTexCoord2f(0.0f, 1.0f); 
		glVertex3f( 10.0f, -10.0f, -10.0f);
		glTexCoord2f(0.0f, 0.0f); 
		glVertex3f( 10.0f, -10.0f,  10.0f);
		glTexCoord2f(1.0f, 0.0f); 
		glVertex3f(-10.0f, -10.0f,  10.0f);
	}
	glEnd();
	glPopMatrix();

	//Skybox right face
	glBindTexture(GL_TEXTURE_2D, Textures[3]);
	glBegin (GL_QUADS);
	{			
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f( 10.0f, -10.0f, -10.0f);
		glTexCoord2f(1.0f, 1.0f); 
		glVertex3f( 10.0f,  10.0f, -10.0f);
		glTexCoord2f(0.0f, 1.0f); 
		glVertex3f( 10.0f,  10.0f,  10.0f);
		glTexCoord2f(0.0f, 0.0f); 
		glVertex3f( 10.0f, -10.0f,  10.0f);
	}
	glEnd();
		
	//Skybox left face
	glBindTexture(GL_TEXTURE_2D, Textures[2]);
	glBegin (GL_QUADS);
	{				
		glTexCoord2f(0.0f, 0.0f); 
		glVertex3f(-10.0f, -10.0f, -10.0f);
		glTexCoord2f(1.0f, 0.0f); 
		glVertex3f(-10.0f, -10.0f,  10.0f);
		glTexCoord2f(1.0f, 1.0f); 
		glVertex3f(-10.0f,  10.0f,  10.0f);
		glTexCoord2f(0.0f, 1.0f); 
		glVertex3f(-10.0f,  10.0f, -10.0f);
	}

	glEnd();//end drawing

	glPopMatrix();

	glEnable(GL_LIGHTING);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glDisable(GL_TEXTURE_2D);
}

///////////////
//Main()
///////////////
void main(int argc, char **argv) 
{ 
		/////////////////////////////////////
	    //Setup any game objects here
		/////////////////////////////////////
		
	    //read in the default curve data
	    ReadCurveData();

		gCamera = new Camera(); //instantiate the camera object

		//instantiate & setup each of the three curves
		gBCurve0 = new BCurve();
		gBCurve1 = new BCurve();
		gBCurve2 = new BCurve();

		SetupDefaultCurve();

		gBCurve0t = new BCurve();
		gBCurve0t->Init();

		gBCurve1t = new BCurve();
		gBCurve1t->Init();

		gBCurve2t = new BCurve();
		gBCurve2t->Init();

		gTeapot = new Teapot();

        GLfloat afPropertiesAmbient [] = {1.0, 1.0, 1.0, 1.00}; 
        GLfloat afPropertiesDiffuse [] = {0.75, 0.75, 0.75, 1.00}; 
        GLfloat afPropertiesSpecular[] = {1.00, 1.00, 1.00, 1.00}; 

        GLsizei iWidth = 1024.0; 
        GLsizei iHeight = 768.0; 
		glutInit(&argc, argv);
		glutInitContextVersion(1, 2);
		//glutInitContextFlags(GLUT_FORWARD_COMPATIBLE);
		//glutInitContextProfile(GLUT_CORE_PROFILE);
		glutInitWindowPosition( 0, 0);
		glutInitWindowSize(iWidth, iHeight);
		glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
        glutCreateWindow( "3D Bezier Curve Generation and Motion" );
        glutDisplayFunc(DrawScene);
        glutIdleFunc(IdleScene);
        glutReshapeFunc(ResizeWindow);
        glutKeyboardFunc(KeyboardInput);

		glewInit();

		//Note: Always init stuff here, otherwise the gl state calls wont work

		//init the skybox
		InitSkybox(); //load the skybox textures

		//setup the camera
		glutSetCursor(GLUT_CURSOR_NONE); 
		gCamera->PositionCamera(0, 40.0f, 100.0f,   0, 0, -1.0f,   0, 1, 0);

        glClearColor (0.1, 0.57, 0.8, 1.0);
        glClearDepth( 1.0 ); 

        glEnable(GL_DEPTH_TEST); 
        glEnable(GL_LIGHTING);
        glPolygonMode(GL_FRONT_AND_BACK, ePolygonMode);

        glLightfv( GL_LIGHT0, GL_AMBIENT,  afPropertiesAmbient); 
        glLightfv( GL_LIGHT0, GL_DIFFUSE,  afPropertiesDiffuse); 
        glLightfv( GL_LIGHT0, GL_SPECULAR, afPropertiesSpecular); 
        glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, 1.0); 

        glEnable( GL_LIGHT0 ); 
		glShadeModel(GL_SMOOTH);										
		glDepthFunc(GL_LEQUAL);								
		glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);							
		glEnable(GL_CULL_FACE);								

        ResizeWindow(iWidth, iHeight); 

        glutMainLoop(); 
}

////////////////////////
//Main update loop
////////////////////////
void IdleScene()
{
	_time = glutGet(GLUT_ELAPSED_TIME);
	delta  = _time - lastTime;

	//Update teapot coords here
	gTeapot->mX = mtPointSet0.x;
	gTeapot->mY = mtPointSet0.y;
	gTeapot->mZ = mtPointSet0.z;

	if(generate)
	{
		for(int i = 0; i < 5; ++i)
		{
			gBCurve0->points[i].x = Lerp(delta * 0.004, gBCurve0->points[i].x, gBCurve0t->points[i].x);
			gBCurve0->points[i].y = Lerp(delta * 0.004, gBCurve0->points[i].y, gBCurve0t->points[i].y);
			gBCurve1->points[i].x = Lerp(delta * 0.004, gBCurve1->points[i].x, gBCurve1t->points[i].x);
			gBCurve1->points[i].y = Lerp(delta * 0.004, gBCurve1->points[i].y, gBCurve1t->points[i].y);
			gBCurve2->points[i].x = Lerp(delta * 0.004, gBCurve2->points[i].x, gBCurve2t->points[i].x);
			gBCurve2->points[i].y = Lerp(delta * 0.004, gBCurve2->points[i].y, gBCurve2t->points[i].y);
		}
	}

	if(stepTime0 < 0.999)
	{
		stepTime1 = 0;
		stepTime2 = 0;
		mtPointSet0 = PointOnSpline(gBCurve0, stepTime0);
		stepTime0 += step * delta;
		if(useGravity)
		{
			CheckSlope(step * delta);
		}
		CalculateSlopeRotation();
	}
	else if(stepTime0 > 0.999 && stepTime1 < 0.999 )
	{
		mtPointSet0 = PointOnSpline(gBCurve1, stepTime1);
		stepTime1 += step * delta;
		if(useGravity)
		{
			CheckSlope(step * delta);
		}
		CalculateSlopeRotation();
	}
	else if(stepTime1 > 0.999 && stepTime2 < 0.999)
	{
		mtPointSet0 = PointOnSpline(gBCurve2, stepTime2);
		stepTime2 += step * delta;
		if(useGravity)
		{
			CheckSlope(step * delta);
		}
		CalculateSlopeRotation();
	}
	else if(stepTime2 > 0.999)
	{
		stepTime0 = 0;
	}
	

	if(useObjectCam)
	{
		gCamera->PositionCamera(Lerp(0.0002 * delta, gCamera->GetPosX(), gTeapot->mX + 10),
								Lerp(0.0002 * delta, gCamera->GetPosY(), gTeapot->mY + 10), 
								Lerp(0.0002 * delta, gCamera->GetPosZ(), gTeapot->mZ + 10),   
								gTeapot->mX,  gTeapot->mY,  gTeapot->mZ,   0, 1, 0);
		
	}

	if(useFrontCam)
	{
		gCamera->PositionCamera(gTeapot->mX,
								gTeapot->mY  + k, 
								gTeapot->mZ,   
								0, rot, 65.0f,   0, 1, 0);
	}

	lastTime = _time;

	DebugInfo();

	glutPostRedisplay();
}

/////////////////////////
//Draw 3D grid plane
/////////////////////////
void Draw3DGrid()
{
	glDisable(GL_LIGHTING);

	glLineWidth(1.0f);
	glColor3f(1.0, 1.0, 1.0);

	glPushMatrix();

	glTranslatef(50.0f, -10.0f, 50.0f);

	for(float i = -50; i <= 50; i += 1)
	{
		glBegin(GL_LINES);

			//along x axis
			glVertex3f(-50, 0, i);
			glVertex3f(50, 0, i);

			//along z axis
			glVertex3f(i, 0, -50);
			glVertex3f(i, 0, 50);

		glEnd();
	}

	glPopMatrix();

	glEnable(GL_LIGHTING);
}

//////////////////////
//Draw the scene
//////////////////////
void DrawScene()
{
	   glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 

	   glLoadIdentity();

	   //Update camera here
	   gCamera->Update();

	   glPushMatrix();

	   	   	//render skybox
	   	    RenderSkybox();

			//render grid
			Draw3DGrid();

			//render the curves
			gBCurve0->DrawCurve(1.0f, 0.0f, 0.0f);
			gBCurve0->DrawCPoints();

			gBCurve1->DrawCurve(0.0f, 1.0f, 0.0f);
			gBCurve1->DrawCPoints();

			gBCurve2->DrawCurve(0.0f, 0.0f, 1.0f);
			gBCurve2->DrawCPoints();
				
		
			//render the teapot
			glPushMatrix();

			glRotatef(Rad2Deg(theta), gTeapot->mX, gTeapot->mY, gTeapot->mZ);

			if(!useFrontCam)
			{
				RenderStaticLight();
				gTeapot->DrawTeapot();
			}
			glPopMatrix();

	  glPopMatrix();

	  glutSwapBuffers(); 
}

void ResizeWindow(GLsizei iWidth, GLsizei iHeight)
{
    glViewport( 0, 0, iWidth, iHeight ); 
    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();

	//calculate aspect ratio
	gluPerspective(45.0f, iWidth/ iHeight, 1.0f, 500.0f);

	glMatrixMode(GL_MODELVIEW);// Select The Modelview Matrix
	glLoadIdentity();// Reset The Modelview Matrix
}

void KeyboardInput(unsigned char cKey, int iX, int iY)
{
	//implement me if needed
	switch(cKey)
	{
	case ' ': //space key
		step = 0.0004f;
		SetupCurves();
		gBCurve0t->Init();
		gBCurve1t->Init();
		gBCurve2t->Init();
		SetupCurvesT();
		generate = true;
		useGravity = false;
		break;

	case 'c': //use this to switch camera to object view (maybe lerp it to there)
		//useObjectCam = !useObjectCam;
		useFrontCam = false;
		useObjectCam = true;
		break;

	case 'f': //use front cam
		useFrontCam = true;
		useObjectCam = false;
		break;

	case 'v':
		useObjectCam = false;
		useFrontCam = false;
		break;

	case 'r':
		//reset to default spline
		 SetupDefaultCurveT();
		 useGravity = true;
		break;
	}

}

